package bug;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.regex.Pattern;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFClientAnchor;
import org.apache.poi.hssf.usermodel.HSSFComment;
import org.apache.poi.hssf.usermodel.HSSFCreationHelper;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFPatriarch;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.ClientAnchor;
import org.apache.poi.ss.usermodel.Comment;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.Drawing;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFComment;
import org.apache.poi.xssf.usermodel.XSSFCreationHelper;
import org.apache.poi.xssf.usermodel.XSSFDrawing;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class Text_xNNNN_applyFont {
	private static final Pattern CTRL_CHARS_ESCAPE_PATTERN = Pattern.compile("(_x[0-9a-fA-F]{4}_)");
	private static final String PRE_CTRL_CHARS_ESCAPE_TEXT = "_x005f";

	public static void main(String[] args) throws IOException {
		testHSSF(); // .xls
		testSXSSF(); // .xlsx
		testXSSF(); // .xlsx
	}
	
	private static void testHSSF() throws IOException {
		
		
		HSSFWorkbook workbook = new HSSFWorkbook();

		HSSFFont font = workbook.createFont();
		font.setFontName("Verdana");
		font.setFontHeightInPoints((short)14);
		font.setBoldweight((short)700);
		font.setColor(IndexedColors.BLUE.getIndex());
		
		// CreationHelper creationHelper = workbook.getCreationHelper();     
		HSSFCreationHelper creationHelper = workbook.getCreationHelper();     
		HSSFSheet sheet = workbook.createSheet();
		HSSFPatriarch drawing = sheet.createDrawingPatriarch();     
		HSSFClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 2, 2, 5, 5); 
		
		HSSFRow row_0 = sheet.createRow(0);
		
		HSSFCell cell_0_0 = row_0.createCell(0);
		cell_0_0.setCellValue("Cell 0:0");
		HSSFComment comment_0_0 = drawing.createCellComment(anchor);
		HSSFRichTextString rts_0_0 = creationHelper.createRichTextString("RichString 0:0 _x0046_");
		try {
		    rts_0_0.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testHSSF: Comment 0:0 - "+iae);
		}
		comment_0_0.setString(rts_0_0);
		cell_0_0.setCellComment(comment_0_0);

		HSSFCell cell_0_1 = row_0.createCell(1);
		cell_0_1.setCellValue("Cell 0:1");
		HSSFComment comment_0_1 = drawing.createCellComment(anchor);
		HSSFRichTextString rts_0_1 = creationHelper.createRichTextString(Utils.preAppendText("RichString 0:1 _x0046_", CTRL_CHARS_ESCAPE_PATTERN, PRE_CTRL_CHARS_ESCAPE_TEXT));
		try {
		    rts_0_1.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testHSSF: Comment 0:1 - "+iae);
		}
		comment_0_1.setString(rts_0_1);
		cell_0_1.setCellComment(comment_0_1);

		HSSFCell cell_0_2 = row_0.createCell(2);
		cell_0_2.setCellValue("Cell 0:2");
		HSSFComment comment_0_2 = drawing.createCellComment(anchor);
		HSSFRichTextString rts_0_2 = creationHelper.createRichTextString(Utils.capitalizeMatchingRegExp("RichString 0:2 _x0046_", CTRL_CHARS_ESCAPE_PATTERN));
		try {
		    rts_0_2.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testHSSF: Comment 0:2 - "+iae);
		}
		comment_0_2.setString(rts_0_2);
		cell_0_2.setCellComment(comment_0_2);
		
		String fname = "Test_xNNNN_applyFont_HSSF.xls";
		FileOutputStream out = new FileOutputStream(fname);
		workbook.write(out);
		out.close();		
	}
	
	private static void testSXSSF() throws IOException {
		SXSSFWorkbook workbook = new SXSSFWorkbook();
		
		Font font = workbook.createFont();
		font.setFontName("Verdana");
		font.setFontHeightInPoints((short)14);
		font.setBoldweight((short)700);
		font.setColor(IndexedColors.BLUE.getIndex());
		
		CreationHelper creationHelper = workbook.getCreationHelper();     
		Sheet sheet = workbook.createSheet();
		Drawing drawing = sheet.createDrawingPatriarch();     
		ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 2, 2, 5, 5); 
		
		Row row_0 = sheet.createRow(0);
		
		Cell cell_0_0 = row_0.createCell(0);
		cell_0_0.setCellValue("Cell 0:0");
		Comment comment_0_0 = drawing.createCellComment(anchor);
		RichTextString rts_0_0 = creationHelper.createRichTextString("RichString 0:0 _x0046_");
		try {
		    rts_0_0.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testSXSSF: Comment 0:0 - "+iae);
		}
		comment_0_0.setString(rts_0_0);
		cell_0_0.setCellComment(comment_0_0);
		comment_0_0.setRow(0);
		comment_0_0.setColumn(0);

		Cell cell_0_1 = row_0.createCell(1);
		cell_0_1.setCellValue("Cell 0:1");
		Comment comment_0_1 = drawing.createCellComment(anchor);
		RichTextString rts_0_1 = creationHelper.createRichTextString(Utils.preAppendText("RichString 0:1 _x0046_", CTRL_CHARS_ESCAPE_PATTERN, PRE_CTRL_CHARS_ESCAPE_TEXT));
		try {
		    rts_0_1.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testSXSSF: Comment 0:1 - "+iae);
		}
		comment_0_1.setString(rts_0_1);
		cell_0_1.setCellComment(comment_0_1);
		comment_0_1.setRow(0);
		comment_0_1.setColumn(1);

		Cell cell_0_2 = row_0.createCell(2);
		cell_0_2.setCellValue("Cell 0:2");
		Comment comment_0_2 = drawing.createCellComment(anchor);
		RichTextString rts_0_2 = creationHelper.createRichTextString(Utils.capitalizeMatchingRegExp("RichString 0:2 _x0046_", CTRL_CHARS_ESCAPE_PATTERN));
		try {
		    rts_0_2.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testSXSSF: Comment 0:2 - "+iae);
		}
		comment_0_2.setString(rts_0_2);
		cell_0_2.setCellComment(comment_0_2);
		comment_0_2.setRow(0);
		comment_0_2.setColumn(2);
		
		String fname = "Test_xNNNN_applyFont_SXSSF.xlsx";
		FileOutputStream out = new FileOutputStream(fname);
		workbook.write(out);
		out.close();		
	}
	
	private static void testXSSF() throws IOException {
		XSSFWorkbook workbook = new XSSFWorkbook();
		
		XSSFFont font = workbook.createFont();
		font.setFontName("Verdana");
		font.setFontHeightInPoints((short)14);
		font.setBoldweight((short)700);
		font.setColor(IndexedColors.BLUE.getIndex());
		
		XSSFCreationHelper creationHelper = workbook.getCreationHelper();     
		XSSFSheet sheet = workbook.createSheet();
		XSSFDrawing drawing = sheet.createDrawingPatriarch();     
		XSSFClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 2, 2, 5, 5); 
		
		XSSFRow row_0 = sheet.createRow(0);
		
		XSSFCell cell_0_0 = row_0.createCell(0);
		cell_0_0.setCellValue("Cell 0:0");
		XSSFComment comment_0_0 = drawing.createCellComment(anchor);
		XSSFRichTextString rts_0_0 = creationHelper.createRichTextString("RichString 0:0 _x0046_");
		try {
		    rts_0_0.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testXSSF: Comment 0:0 - "+iae);
		}
		comment_0_0.setString(rts_0_0);
		cell_0_0.setCellComment(comment_0_0);

		XSSFCell cell_0_1 = row_0.createCell(1);
		cell_0_1.setCellValue("Cell 0:1");
		XSSFComment comment_0_1 = drawing.createCellComment(anchor);
		// XSSFRichTextString rts_0_1 = creationHelper.createRichTextString(Utils.preAppendText("RichString 0:1 _x0046_", CTRL_CHARS_ESCAPE_PATTERN, PRE_CTRL_CHARS_ESCAPE_TEXT));
		RichTextString rts_0_1 = creationHelper.createRichTextString(Utils.preAppendText("RichString 0:1 _x0046_", CTRL_CHARS_ESCAPE_PATTERN, PRE_CTRL_CHARS_ESCAPE_TEXT));
		try {
		    rts_0_1.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testXSSF: Comment 0:1 - "+iae);
		}
		comment_0_1.setString(rts_0_1);
		cell_0_1.setCellComment(comment_0_1);
		
		XSSFCell cell_0_2 = row_0.createCell(2);
		cell_0_2.setCellValue("Cell 0:2");
		XSSFComment comment_0_2 = drawing.createCellComment(anchor);
		XSSFRichTextString rts_0_2 = creationHelper.createRichTextString(Utils.capitalizeMatchingRegExp("RichString 0:2 _x0046_", CTRL_CHARS_ESCAPE_PATTERN));
		try {
		    rts_0_2.applyFont(font);
		} catch(java.lang.IllegalArgumentException iae) {
			System.out.println("testXSSF: Comment 0:2 - "+iae);
		}
		comment_0_2.setString(rts_0_2);
		cell_0_2.setCellComment(comment_0_2);
		
		String fname = "Test_xNNNN_applyFont_XSSF.xlsx";
		FileOutputStream out = new FileOutputStream(fname);
		workbook.write(out);
		out.close();		
	}
	
}
