package bug;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Utils {
	/**
	 * Append a text before a substring matching a pattern
	 * @param text is the text to search for the substrings
	 * @param pattern is the pattern which has to match the substring
	 * @param  pre_text is the text to append before the subtring matching the pattern
	 * @return changed text
	 */
	public static String preAppendText(String text, Pattern pattern, String pre_text) {
		if(text == null || text.isEmpty() || pattern == null || pre_text == null || pre_text.isEmpty()) {
			return text;
		}
		StringBuffer result = new StringBuffer();
		Matcher matcher = pattern.matcher(text);
		while (matcher.find()) {
			matcher.appendReplacement(result, pre_text+matcher.group(0));
		}
		matcher.appendTail(result);
		return result.toString();
	}
	
	/**
	 * Remove a text before a substring matching a pattern
	 * @param text is the text to search for the substrings
	 * @param pattern is the pattern which has to match the substring
	 * @return changed text
	 */
	public static String removePreAppendText(String text, Pattern pattern) {
		if(text == null || text.isEmpty() || pattern == null) {
			return text;
		}
		Matcher m = pattern.matcher(text);
		StringBuffer sb = new StringBuffer(text.length());
		while (m.find()) {
			m.appendReplacement(sb, Matcher.quoteReplacement(m.group(1)));
		}
		m.appendTail(sb);
		return sb.toString();
	}	
	
	/**
	 * Set uppercase a substring matching a pattern
	 * @param text is the text to search for the substrings
	 * @param pattern is the pattern which has to match the substring
	 * @return changed text
	 */
	public static String capitalizeMatchingRegExp(String text, Pattern pattern) {
		if(text == null || text.isEmpty() || pattern == null) {
			return text;
		}
		StringBuilder sb= new StringBuilder(text);
		Matcher matcher = pattern.matcher(sb);
		while (matcher.find()) {
			String buf= sb.substring(matcher.start(), matcher.end()).toUpperCase();
			sb.replace(matcher.start(), matcher.end(), buf);
			matcher.reset(); // if text = "_x005F_x0046_", then without reset the new string in "_X005F_x0046_", with reset it is "_X005F_X0046_", which is the required purpose of this method.
		}
		return sb.toString();
	}

}
